# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: diff.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import lint_pb2 as lint__pb2
from . import shared_pb2 as shared__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\ndiff.proto\x12\x06gitaly\x1a\nlint.proto\x1a\x0cshared.proto\"\xd8\x06\n\x11\x43ommitDiffRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x16\n\x0eleft_commit_id\x18\x02 \x01(\t\x12\x17\n\x0fright_commit_id\x18\x03 \x01(\t\x12\r\n\x05paths\x18\x05 \x03(\x0c\x12\x16\n\x0e\x63ollapse_diffs\x18\x06 \x01(\x08\x12\x16\n\x0e\x65nforce_limits\x18\x07 \x01(\x08\x12\x11\n\tmax_files\x18\x08 \x01(\x05\x12\x11\n\tmax_lines\x18\t \x01(\x05\x12\x11\n\tmax_bytes\x18\n \x01(\x05\x12\x17\n\x0fmax_patch_bytes\x18\x0e \x01(\x05\x12\x16\n\x0esafe_max_files\x18\x0b \x01(\x05\x12\x16\n\x0esafe_max_lines\x18\x0c \x01(\x05\x12\x16\n\x0esafe_max_bytes\x18\r \x01(\x05\x12\x35\n\tdiff_mode\x18\x0f \x01(\x0e\x32\".gitaly.CommitDiffRequest.DiffMode\x12h\n\"max_patch_bytes_for_file_extension\x18\x10 \x03(\x0b\x32<.gitaly.CommitDiffRequest.MaxPatchBytesForFileExtensionEntry\x12G\n\x12whitespace_changes\x18\x11 \x01(\x0e\x32+.gitaly.CommitDiffRequest.WhitespaceChanges\x12\x19\n\x11\x63ollect_all_paths\x18\x12 \x01(\x08\x1a\x44\n\"MaxPatchBytesForFileExtensionEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\"%\n\x08\x44iffMode\x12\x0b\n\x07\x44\x45\x46\x41ULT\x10\x00\x12\x0c\n\x08WORDDIFF\x10\x01\"y\n\x11WhitespaceChanges\x12\"\n\x1eWHITESPACE_CHANGES_UNSPECIFIED\x10\x00\x12\x1d\n\x19WHITESPACE_CHANGES_IGNORE\x10\x01\x12!\n\x1dWHITESPACE_CHANGES_IGNORE_ALL\x10\x02J\x04\x08\x04\x10\x05R\x18ignore_whitespace_change\"\xff\x01\n\x12\x43ommitDiffResponse\x12\x11\n\tfrom_path\x18\x01 \x01(\x0c\x12\x0f\n\x07to_path\x18\x02 \x01(\x0c\x12\x0f\n\x07\x66rom_id\x18\x03 \x01(\t\x12\r\n\x05to_id\x18\x04 \x01(\t\x12\x10\n\x08old_mode\x18\x05 \x01(\x05\x12\x10\n\x08new_mode\x18\x06 \x01(\x05\x12\x0e\n\x06\x62inary\x18\x07 \x01(\x08\x12\x16\n\x0eraw_patch_data\x18\t \x01(\x0c\x12\x14\n\x0c\x65nd_of_patch\x18\n \x01(\x08\x12\x17\n\x0foverflow_marker\x18\x0b \x01(\x08\x12\x11\n\tcollapsed\x18\x0c \x01(\x08\x12\x11\n\ttoo_large\x18\r \x01(\x08J\x04\x08\x08\x10\t\"\x82\x01\n\x12\x43ommitDeltaRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x16\n\x0eleft_commit_id\x18\x02 \x01(\t\x12\x17\n\x0fright_commit_id\x18\x03 \x01(\t\x12\r\n\x05paths\x18\x04 \x03(\x0c\"u\n\x0b\x43ommitDelta\x12\x11\n\tfrom_path\x18\x01 \x01(\x0c\x12\x0f\n\x07to_path\x18\x02 \x01(\x0c\x12\x0f\n\x07\x66rom_id\x18\x03 \x01(\t\x12\r\n\x05to_id\x18\x04 \x01(\t\x12\x10\n\x08old_mode\x18\x05 \x01(\x05\x12\x10\n\x08new_mode\x18\x06 \x01(\x05\":\n\x13\x43ommitDeltaResponse\x12#\n\x06\x64\x65ltas\x18\x01 \x03(\x0b\x32\x13.gitaly.CommitDelta\"o\n\x0eRawDiffRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x16\n\x0eleft_commit_id\x18\x02 \x01(\t\x12\x17\n\x0fright_commit_id\x18\x03 \x01(\t\"\x1f\n\x0fRawDiffResponse\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\"p\n\x0fRawPatchRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x16\n\x0eleft_commit_id\x18\x02 \x01(\t\x12\x17\n\x0fright_commit_id\x18\x03 \x01(\t\" \n\x10RawPatchResponse\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\"q\n\x10\x44iffStatsRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x16\n\x0eleft_commit_id\x18\x02 \x01(\t\x12\x17\n\x0fright_commit_id\x18\x03 \x01(\t\"Q\n\tDiffStats\x12\x0c\n\x04path\x18\x01 \x01(\x0c\x12\x11\n\tadditions\x18\x02 \x01(\x05\x12\x11\n\tdeletions\x18\x03 \x01(\x05\x12\x10\n\x08old_path\x18\x04 \x01(\x0c\"5\n\x11\x44iffStatsResponse\x12 \n\x05stats\x18\x01 \x03(\x0b\x32\x11.gitaly.DiffStats\"\xda\x05\n\x17\x46indChangedPathsRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x13\n\x07\x63ommits\x18\x02 \x03(\tB\x02\x18\x01\x12\x39\n\x08requests\x18\x03 \x03(\x0b\x32\'.gitaly.FindChangedPathsRequest.Request\x12S\n\x16merge_commit_diff_mode\x18\x04 \x01(\x0e\x32\x33.gitaly.FindChangedPathsRequest.MergeCommitDiffMode\x12\x14\n\x0c\x66ind_renames\x18\x05 \x01(\x08\x1a\xc2\x02\n\x07Request\x12K\n\x0ctree_request\x18\x01 \x01(\x0b\x32\x33.gitaly.FindChangedPathsRequest.Request.TreeRequestH\x00\x12O\n\x0e\x63ommit_request\x18\x02 \x01(\x0b\x32\x35.gitaly.FindChangedPathsRequest.Request.CommitRequestH\x00\x1a\x46\n\x0bTreeRequest\x12\x1a\n\x12left_tree_revision\x18\x01 \x01(\t\x12\x1b\n\x13right_tree_revision\x18\x02 \x01(\t\x1aI\n\rCommitRequest\x12\x17\n\x0f\x63ommit_revision\x18\x01 \x01(\t\x12\x1f\n\x17parent_commit_revisions\x18\x02 \x03(\tB\x06\n\x04type\"\x90\x01\n\x13MergeCommitDiffMode\x12&\n\"MERGE_COMMIT_DIFF_MODE_UNSPECIFIED\x10\x00\x12)\n%MERGE_COMMIT_DIFF_MODE_INCLUDE_MERGES\x10\x01\x12&\n\"MERGE_COMMIT_DIFF_MODE_ALL_PARENTS\x10\x02\"?\n\x18\x46indChangedPathsResponse\x12#\n\x05paths\x18\x01 \x03(\x0b\x32\x14.gitaly.ChangedPaths\"\x83\x02\n\x0c\x43hangedPaths\x12\x0c\n\x04path\x18\x01 \x01(\x0c\x12+\n\x06status\x18\x02 \x01(\x0e\x32\x1b.gitaly.ChangedPaths.Status\x12\x10\n\x08old_mode\x18\x03 \x01(\x05\x12\x10\n\x08new_mode\x18\x04 \x01(\x05\x12\x13\n\x0bold_blob_id\x18\x05 \x01(\t\x12\x13\n\x0bnew_blob_id\x18\x06 \x01(\t\x12\x10\n\x08old_path\x18\x07 \x01(\x0c\"X\n\x06Status\x12\t\n\x05\x41\x44\x44\x45\x44\x10\x00\x12\x0c\n\x08MODIFIED\x10\x01\x12\x0b\n\x07\x44\x45LETED\x10\x02\x12\x0f\n\x0bTYPE_CHANGE\x10\x03\x12\n\n\x06\x43OPIED\x10\x04\x12\x0b\n\x07RENAMED\x10\x05\"m\n\x11GetPatchIDRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x14\n\x0cold_revision\x18\x02 \x01(\x0c\x12\x14\n\x0cnew_revision\x18\x03 \x01(\x0c\"&\n\x12GetPatchIDResponse\x12\x10\n\x08patch_id\x18\x01 \x01(\t\"+\n\tRangePair\x12\x0e\n\x06range1\x18\x01 \x01(\t\x12\x0e\n\x06range2\x18\x02 \x01(\t\"+\n\rRevisionRange\x12\x0c\n\x04rev1\x18\x01 \x01(\t\x12\x0c\n\x04rev2\x18\x02 \x01(\t\"=\n\x11\x42\x61seWithRevisions\x12\x0c\n\x04\x62\x61se\x18\x01 \x01(\t\x12\x0c\n\x04rev1\x18\x02 \x01(\t\x12\x0c\n\x04rev2\x18\x03 \x01(\t\"\xe5\x01\n\x13RawRangeDiffRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\'\n\nrange_pair\x18\x02 \x01(\x0b\x32\x11.gitaly.RangePairH\x00\x12/\n\x0erevision_range\x18\x03 \x01(\x0b\x32\x15.gitaly.RevisionRangeH\x00\x12\x38\n\x13\x62\x61se_with_revisions\x18\x04 \x01(\x0b\x32\x19.gitaly.BaseWithRevisionsH\x00\x42\x0c\n\nrange_spec\"$\n\x14RawRangeDiffResponse\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\"\xe2\x01\n\x10RangeDiffRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\'\n\nrange_pair\x18\x02 \x01(\x0b\x32\x11.gitaly.RangePairH\x00\x12/\n\x0erevision_range\x18\x03 \x01(\x0b\x32\x15.gitaly.RevisionRangeH\x00\x12\x38\n\x13\x62\x61se_with_revisions\x18\x04 \x01(\x0b\x32\x19.gitaly.BaseWithRevisionsH\x00\x42\x0c\n\nrange_spec\"\xc4\x02\n\x11RangeDiffResponse\x12\x16\n\x0e\x66rom_commit_id\x18\x01 \x01(\t\x12\x14\n\x0cto_commit_id\x18\x02 \x01(\t\x12\x38\n\ncomparison\x18\x03 \x01(\x0e\x32$.gitaly.RangeDiffResponse.Comparator\x12\x1c\n\x14\x63ommit_message_title\x18\x07 \x01(\t\x12\x12\n\npatch_data\x18\x08 \x01(\x0c\x12\x14\n\x0c\x65nd_of_patch\x18\t \x01(\x08\"\x7f\n\nComparator\x12 \n\x1c\x43OMPARATOR_EQUAL_UNSPECIFIED\x10\x00\x12\x1b\n\x17\x43OMPARATOR_GREATER_THAN\x10\x01\x12\x18\n\x14\x43OMPARATOR_LESS_THAN\x10\x02\x12\x18\n\x14\x43OMPARATOR_NOT_EQUAL\x10\x03\"\xdc\x03\n\x10\x44iffBlobsRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x35\n\nblob_pairs\x18\x02 \x03(\x0b\x32!.gitaly.DiffBlobsRequest.BlobPair\x12\x34\n\tdiff_mode\x18\x03 \x01(\x0e\x32!.gitaly.DiffBlobsRequest.DiffMode\x12\x46\n\x12whitespace_changes\x18\x04 \x01(\x0e\x32*.gitaly.DiffBlobsRequest.WhitespaceChanges\x1a/\n\x08\x42lobPair\x12\x10\n\x08left_oid\x18\x01 \x01(\t\x12\x11\n\tright_oid\x18\x02 \x01(\t\"9\n\x08\x44iffMode\x12\x19\n\x15\x44IFF_MODE_UNSPECIFIED\x10\x00\x12\x12\n\x0e\x44IFF_MODE_WORD\x10\x01\"y\n\x11WhitespaceChanges\x12\"\n\x1eWHITESPACE_CHANGES_UNSPECIFIED\x10\x00\x12\x1d\n\x19WHITESPACE_CHANGES_IGNORE\x10\x01\x12!\n\x1dWHITESPACE_CHANGES_IGNORE_ALL\x10\x02\"\xcb\x01\n\x11\x44iffBlobsResponse\x12\x14\n\x0cleft_blob_id\x18\x01 \x01(\t\x12\x15\n\rright_blob_id\x18\x02 \x01(\t\x12\r\n\x05patch\x18\x03 \x01(\x0c\x12\x30\n\x06status\x18\x04 \x01(\x0e\x32 .gitaly.DiffBlobsResponse.Status\x12\x0e\n\x06\x62inary\x18\x05 \x01(\x08\"8\n\x06Status\x12\x15\n\x11STATUS_INCOMPLETE\x10\x00\x12\x17\n\x13STATUS_END_OF_PATCH\x10\x01\x32\xa4\x06\n\x0b\x44iffService\x12M\n\nCommitDiff\x12\x19.gitaly.CommitDiffRequest\x1a\x1a.gitaly.CommitDiffResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12P\n\x0b\x43ommitDelta\x12\x1a.gitaly.CommitDeltaRequest\x1a\x1b.gitaly.CommitDeltaResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12\x44\n\x07RawDiff\x12\x16.gitaly.RawDiffRequest\x1a\x17.gitaly.RawDiffResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12G\n\x08RawPatch\x12\x17.gitaly.RawPatchRequest\x1a\x18.gitaly.RawPatchResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12J\n\tDiffStats\x12\x18.gitaly.DiffStatsRequest\x1a\x19.gitaly.DiffStatsResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12_\n\x10\x46indChangedPaths\x12\x1f.gitaly.FindChangedPathsRequest\x1a .gitaly.FindChangedPathsResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12K\n\nGetPatchID\x12\x19.gitaly.GetPatchIDRequest\x1a\x1a.gitaly.GetPatchIDResponse\"\x06\xfa\x97(\x02\x08\x02\x12S\n\x0cRawRangeDiff\x12\x1b.gitaly.RawRangeDiffRequest\x1a\x1c.gitaly.RawRangeDiffResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12J\n\tRangeDiff\x12\x18.gitaly.RangeDiffRequest\x1a\x19.gitaly.RangeDiffResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12J\n\tDiffBlobs\x12\x18.gitaly.DiffBlobsRequest\x1a\x19.gitaly.DiffBlobsResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x42\x34Z2gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'diff_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z2gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypb'
  _COMMITDIFFREQUEST_MAXPATCHBYTESFORFILEEXTENSIONENTRY._options = None
  _COMMITDIFFREQUEST_MAXPATCHBYTESFORFILEEXTENSIONENTRY._serialized_options = b'8\001'
  _COMMITDIFFREQUEST.fields_by_name['repository']._options = None
  _COMMITDIFFREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _COMMITDELTAREQUEST.fields_by_name['repository']._options = None
  _COMMITDELTAREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _RAWDIFFREQUEST.fields_by_name['repository']._options = None
  _RAWDIFFREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _RAWPATCHREQUEST.fields_by_name['repository']._options = None
  _RAWPATCHREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _DIFFSTATSREQUEST.fields_by_name['repository']._options = None
  _DIFFSTATSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FINDCHANGEDPATHSREQUEST.fields_by_name['repository']._options = None
  _FINDCHANGEDPATHSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FINDCHANGEDPATHSREQUEST.fields_by_name['commits']._options = None
  _FINDCHANGEDPATHSREQUEST.fields_by_name['commits']._serialized_options = b'\030\001'
  _GETPATCHIDREQUEST.fields_by_name['repository']._options = None
  _GETPATCHIDREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _RAWRANGEDIFFREQUEST.fields_by_name['repository']._options = None
  _RAWRANGEDIFFREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _RANGEDIFFREQUEST.fields_by_name['repository']._options = None
  _RANGEDIFFREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _DIFFBLOBSREQUEST.fields_by_name['repository']._options = None
  _DIFFBLOBSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _DIFFSERVICE.methods_by_name['CommitDiff']._options = None
  _DIFFSERVICE.methods_by_name['CommitDiff']._serialized_options = b'\372\227(\002\010\002'
  _DIFFSERVICE.methods_by_name['CommitDelta']._options = None
  _DIFFSERVICE.methods_by_name['CommitDelta']._serialized_options = b'\372\227(\002\010\002'
  _DIFFSERVICE.methods_by_name['RawDiff']._options = None
  _DIFFSERVICE.methods_by_name['RawDiff']._serialized_options = b'\372\227(\002\010\002'
  _DIFFSERVICE.methods_by_name['RawPatch']._options = None
  _DIFFSERVICE.methods_by_name['RawPatch']._serialized_options = b'\372\227(\002\010\002'
  _DIFFSERVICE.methods_by_name['DiffStats']._options = None
  _DIFFSERVICE.methods_by_name['DiffStats']._serialized_options = b'\372\227(\002\010\002'
  _DIFFSERVICE.methods_by_name['FindChangedPaths']._options = None
  _DIFFSERVICE.methods_by_name['FindChangedPaths']._serialized_options = b'\372\227(\002\010\002'
  _DIFFSERVICE.methods_by_name['GetPatchID']._options = None
  _DIFFSERVICE.methods_by_name['GetPatchID']._serialized_options = b'\372\227(\002\010\002'
  _DIFFSERVICE.methods_by_name['RawRangeDiff']._options = None
  _DIFFSERVICE.methods_by_name['RawRangeDiff']._serialized_options = b'\372\227(\002\010\002'
  _DIFFSERVICE.methods_by_name['RangeDiff']._options = None
  _DIFFSERVICE.methods_by_name['RangeDiff']._serialized_options = b'\372\227(\002\010\002'
  _DIFFSERVICE.methods_by_name['DiffBlobs']._options = None
  _DIFFSERVICE.methods_by_name['DiffBlobs']._serialized_options = b'\372\227(\002\010\002'
  _globals['_COMMITDIFFREQUEST']._serialized_start=49
  _globals['_COMMITDIFFREQUEST']._serialized_end=905
  _globals['_COMMITDIFFREQUEST_MAXPATCHBYTESFORFILEEXTENSIONENTRY']._serialized_start=643
  _globals['_COMMITDIFFREQUEST_MAXPATCHBYTESFORFILEEXTENSIONENTRY']._serialized_end=711
  _globals['_COMMITDIFFREQUEST_DIFFMODE']._serialized_start=713
  _globals['_COMMITDIFFREQUEST_DIFFMODE']._serialized_end=750
  _globals['_COMMITDIFFREQUEST_WHITESPACECHANGES']._serialized_start=752
  _globals['_COMMITDIFFREQUEST_WHITESPACECHANGES']._serialized_end=873
  _globals['_COMMITDIFFRESPONSE']._serialized_start=908
  _globals['_COMMITDIFFRESPONSE']._serialized_end=1163
  _globals['_COMMITDELTAREQUEST']._serialized_start=1166
  _globals['_COMMITDELTAREQUEST']._serialized_end=1296
  _globals['_COMMITDELTA']._serialized_start=1298
  _globals['_COMMITDELTA']._serialized_end=1415
  _globals['_COMMITDELTARESPONSE']._serialized_start=1417
  _globals['_COMMITDELTARESPONSE']._serialized_end=1475
  _globals['_RAWDIFFREQUEST']._serialized_start=1477
  _globals['_RAWDIFFREQUEST']._serialized_end=1588
  _globals['_RAWDIFFRESPONSE']._serialized_start=1590
  _globals['_RAWDIFFRESPONSE']._serialized_end=1621
  _globals['_RAWPATCHREQUEST']._serialized_start=1623
  _globals['_RAWPATCHREQUEST']._serialized_end=1735
  _globals['_RAWPATCHRESPONSE']._serialized_start=1737
  _globals['_RAWPATCHRESPONSE']._serialized_end=1769
  _globals['_DIFFSTATSREQUEST']._serialized_start=1771
  _globals['_DIFFSTATSREQUEST']._serialized_end=1884
  _globals['_DIFFSTATS']._serialized_start=1886
  _globals['_DIFFSTATS']._serialized_end=1967
  _globals['_DIFFSTATSRESPONSE']._serialized_start=1969
  _globals['_DIFFSTATSRESPONSE']._serialized_end=2022
  _globals['_FINDCHANGEDPATHSREQUEST']._serialized_start=2025
  _globals['_FINDCHANGEDPATHSREQUEST']._serialized_end=2755
  _globals['_FINDCHANGEDPATHSREQUEST_REQUEST']._serialized_start=2286
  _globals['_FINDCHANGEDPATHSREQUEST_REQUEST']._serialized_end=2608
  _globals['_FINDCHANGEDPATHSREQUEST_REQUEST_TREEREQUEST']._serialized_start=2455
  _globals['_FINDCHANGEDPATHSREQUEST_REQUEST_TREEREQUEST']._serialized_end=2525
  _globals['_FINDCHANGEDPATHSREQUEST_REQUEST_COMMITREQUEST']._serialized_start=2527
  _globals['_FINDCHANGEDPATHSREQUEST_REQUEST_COMMITREQUEST']._serialized_end=2600
  _globals['_FINDCHANGEDPATHSREQUEST_MERGECOMMITDIFFMODE']._serialized_start=2611
  _globals['_FINDCHANGEDPATHSREQUEST_MERGECOMMITDIFFMODE']._serialized_end=2755
  _globals['_FINDCHANGEDPATHSRESPONSE']._serialized_start=2757
  _globals['_FINDCHANGEDPATHSRESPONSE']._serialized_end=2820
  _globals['_CHANGEDPATHS']._serialized_start=2823
  _globals['_CHANGEDPATHS']._serialized_end=3082
  _globals['_CHANGEDPATHS_STATUS']._serialized_start=2994
  _globals['_CHANGEDPATHS_STATUS']._serialized_end=3082
  _globals['_GETPATCHIDREQUEST']._serialized_start=3084
  _globals['_GETPATCHIDREQUEST']._serialized_end=3193
  _globals['_GETPATCHIDRESPONSE']._serialized_start=3195
  _globals['_GETPATCHIDRESPONSE']._serialized_end=3233
  _globals['_RANGEPAIR']._serialized_start=3235
  _globals['_RANGEPAIR']._serialized_end=3278
  _globals['_REVISIONRANGE']._serialized_start=3280
  _globals['_REVISIONRANGE']._serialized_end=3323
  _globals['_BASEWITHREVISIONS']._serialized_start=3325
  _globals['_BASEWITHREVISIONS']._serialized_end=3386
  _globals['_RAWRANGEDIFFREQUEST']._serialized_start=3389
  _globals['_RAWRANGEDIFFREQUEST']._serialized_end=3618
  _globals['_RAWRANGEDIFFRESPONSE']._serialized_start=3620
  _globals['_RAWRANGEDIFFRESPONSE']._serialized_end=3656
  _globals['_RANGEDIFFREQUEST']._serialized_start=3659
  _globals['_RANGEDIFFREQUEST']._serialized_end=3885
  _globals['_RANGEDIFFRESPONSE']._serialized_start=3888
  _globals['_RANGEDIFFRESPONSE']._serialized_end=4212
  _globals['_RANGEDIFFRESPONSE_COMPARATOR']._serialized_start=4085
  _globals['_RANGEDIFFRESPONSE_COMPARATOR']._serialized_end=4212
  _globals['_DIFFBLOBSREQUEST']._serialized_start=4215
  _globals['_DIFFBLOBSREQUEST']._serialized_end=4691
  _globals['_DIFFBLOBSREQUEST_BLOBPAIR']._serialized_start=4462
  _globals['_DIFFBLOBSREQUEST_BLOBPAIR']._serialized_end=4509
  _globals['_DIFFBLOBSREQUEST_DIFFMODE']._serialized_start=4511
  _globals['_DIFFBLOBSREQUEST_DIFFMODE']._serialized_end=4568
  _globals['_DIFFBLOBSREQUEST_WHITESPACECHANGES']._serialized_start=752
  _globals['_DIFFBLOBSREQUEST_WHITESPACECHANGES']._serialized_end=873
  _globals['_DIFFBLOBSRESPONSE']._serialized_start=4694
  _globals['_DIFFBLOBSRESPONSE']._serialized_end=4897
  _globals['_DIFFBLOBSRESPONSE_STATUS']._serialized_start=4841
  _globals['_DIFFBLOBSRESPONSE_STATUS']._serialized_end=4897
  _globals['_DIFFSERVICE']._serialized_start=4900
  _globals['_DIFFSERVICE']._serialized_end=5704
# @@protoc_insertion_point(module_scope)



# Copyright 2023 Georges Racinet <georges.racinet@octobus.net>
#
# This software may be used and distributed according to the terms of the
# GNU General Public License version 2 or any later version.
#
# SPDX-License-Identifier: GPL-2.0-or-later
"""Tests for OperationService (most mutations)

Because it is not convenient to mutate the Mercurial and Git repositories
Without side-effects, these tests are mostly checking assumptions on the
behaviour of Gitaly
"""
from pathlib import Path
import pytest

from grpc import RpcError

from hgitaly.stub.shared_pb2 import User

from hgitaly.stub.mercurial_operations_pb2 import (
    MercurialPermissions,
    MergeBranchError,
    MergeBranchRequest,
)
from hgitaly.stub.mercurial_operations_pb2_grpc import (
    MercurialOperationsServiceStub,
)
from hgitaly.stub.errors_pb2 import (
    MergeConflictError,
    ReferenceUpdateError,
)
from hgitaly.stub.operations_pb2 import (
    OperationBranchUpdate,
    UserFFBranchRequest,
    UserMergeBranchError,
    UserMergeBranchRequest,
    UserSquashRequest,
)
from hgitaly.stub.operations_pb2_grpc import (
    OperationServiceStub,
)

from . import skip_comparison_tests
if skip_comparison_tests():  # pragma no cover
    pytestmark = pytest.mark.skip

parametrize = pytest.mark.parametrize

TESTS_DATA_DIR = Path(__file__).parent / 'data'
TIP_TAG_NAME = b'tip'
PUBLISH_PERM = MercurialPermissions.PUBLISH


@pytest.fixture
def comparison(gitaly_comparison):
    gitaly_comparison.user = User(gl_id='user-37',
                                  name=b'Test User',
                                  email=b'testuser@heptapod.test',
                                  gl_username='testuser')
    yield gitaly_comparison


def test_compare_squash(comparison):
    """This test is mostly about error cases, as comparing the """
    fixture = comparison
    git_repo = fixture.git_repo
    wrapper = fixture.hg_repo_wrapper

    gl_branch = b'branch/default'
    gl_topic = b'topic/default/sampletop'
    wrapper.write_commit('foo')
    wrapper.write_commit('zoo', topic='sampletop')
    wrapper.write_commit('bar')

    rpc_helper = fixture.rpc_helper(stub_cls=OperationServiceStub,
                                    method_name='UserSquash',
                                    request_cls=UserSquashRequest,
                                    request_defaults=dict(
                                        user=fixture.user,
                                        author=fixture.user,
                                        commit_message=b'squashed'),
                                    request_sha_attrs=['start_sha', 'end_sha'])
    assert_compare_errors = rpc_helper.assert_compare_errors
    # proof that any valid revspec can be passed as "sha" arguments
    git_resp = rpc_helper.call_git_only(start_sha=gl_branch, end_sha=gl_topic)
    squashed_sha = git_resp.squash_sha
    # Gitaly does not move the ref, apparently
    assert git_repo.commit_hash_title(squashed_sha) == [
        squashed_sha.encode('ascii'),
        b'squashed'
    ]

    # revision resolution errors
    assert_compare_errors(start_sha='unresolvable', end_sha=gl_topic)
    assert_compare_errors(start_sha=gl_branch, end_sha='unresolvable')

    # errors on missing arguments
    assert_compare_errors(start_sha=gl_topic)
    assert_compare_errors(end_sha=gl_topic)
    assert_compare_errors(user=None, start_sha=gl_branch, end_sha=gl_topic)
    assert_compare_errors(commit_message=None,
                          start_sha=gl_branch, end_sha=gl_topic)
    assert_compare_errors(author=None, start_sha=gl_branch, end_sha=gl_topic)


def test_compare_ff_branch(comparison):
    """Mostly testing error cases, as comparing results is unpractical.

    Once the Mercurial fast-forward is done, there is nothing to be done
    on the Git side.
    """
    fixture = comparison
    git_repo = fixture.git_repo
    wrapper = fixture.hg_repo_wrapper

    gl_branch = b'branch/default'
    gl_topic = b'topic/default/sampletop'
    ctx0 = wrapper.write_commit('foo')
    sha0 = ctx0.hex()
    sha1 = wrapper.write_commit('foo').hex()
    ctx2 = wrapper.write_commit('zoo', topic='sampletop')
    sha2 = ctx2.hex()
    git_sha2 = git_repo.branches()[gl_topic]['sha']
    ctx_old = wrapper.write_commit('old', parent=ctx0, topic='needs-rebase')

    rpc_helper = fixture.rpc_helper(
        stub_cls=OperationServiceStub,
        method_name='UserFFBranch',
        error_details_normalizer=lambda s, **kw: s.lower(),
        request_cls=UserFFBranchRequest,
        request_defaults=dict(user=fixture.user),
        request_sha_attrs=['commit_id',
                           'expected_old_oid'],
    )
    assert_compare = rpc_helper.assert_compare
    assert_compare_errors = rpc_helper.assert_compare_errors

    # A working Gitaly call, as a baseline that cannot be directly compared.
    git_resp = rpc_helper.call_git_only(branch=gl_branch,
                                        commit_id=sha2,
                                        expected_old_oid=sha1,
                                        )
    assert git_resp.branch_update == OperationBranchUpdate(
        commit_id=git_sha2,
        repo_created=False,
        branch_created=False,
    )
    assert git_repo.commit_hash_title('branch/default')[0] == git_sha2

    # revision resolution errors
    unknown_sha = '2134cafe' * 5
    assert_compare_errors(commit_id=unknown_sha,
                          expected_old_oid=sha0.decode(),
                          branch=gl_branch)
    assert_compare_errors(commit_id='not-an-id',
                          same_details=False,
                          expected_old_oid=sha0.decode(),
                          branch=gl_branch)
    assert_compare_errors(commit_id=sha2,
                          same_details=False,
                          expected_old_oid='not-an-id',
                          branch=gl_branch)
    assert_compare_errors(commit_id=sha2,
                          expected_old_oid=unknown_sha,
                          branch=gl_branch)

    # expected_old_oid mismatch
    assert_compare(commit_id=sha2,
                   expected_old_oid=sha0,
                   branch=gl_branch)

    # errors on missing arguments
    assert_compare_errors(commit_id=sha2)
    assert_compare_errors(user=None, branch=gl_branch, commit_id=sha2)

    # Not a fast-forward
    assert_compare_errors(commit_id=ctx_old.hex().decode(),
                          branch=gl_branch)


def test_compare_user_merge_branch(comparison):
    """Mostly testing error cases, as comparing results is unpractical.

    Once the Mercurial merge is done, there is nothing to be done
    on the Git side, and it is not clear it would give the very same hash.
    """
    fixture = comparison
    wrapper = fixture.hg_repo_wrapper

    rpc_helper = fixture.rpc_helper(
        stub_cls=MercurialOperationsServiceStub,
        method_name='MergeBranch',
        error_details_normalizer=lambda s, **kw: s.lower(),
        request_cls=MergeBranchRequest,
        request_defaults=dict(user=fixture.user, hg_perms=PUBLISH_PERM),
        request_sha_attrs=['commit_id',
                           'expected_old_oid'],
    )

    git_stub = OperationServiceStub(comparison.gitaly_channel)

    def git_merge(**git_kwargs):
        # TODO second request but we may not need it
        del git_kwargs['hg_perms']
        resps = git_stub.UserMergeBranch(iter([
            UserMergeBranchRequest(**git_kwargs),
            UserMergeBranchRequest(apply=True, **git_kwargs)
        ]))
        return [r for r in resps]

    def error_to_git(merge_error):
        if merge_error.HasField('conflict'):
            conflict = merge_error.conflict
            git_conflict = MergeConflictError(
                conflicting_files=conflict.conflicting_files,
                conflicting_commit_ids=[
                    rpc_helper.hg2git(sha)
                    for sha in conflict.conflicting_commit_ids
                ]
            )
            return UserMergeBranchError(merge_conflict=git_conflict)

        if merge_error.HasField('reference_check'):
            check_error = merge_error.reference_check
            git_ref_error = ReferenceUpdateError(
                reference_name=check_error.reference_name,
                old_oid=rpc_helper.hg2git(check_error.old_oid),
                new_oid='',  # see git_error_normalizer()
            )
            return UserMergeBranchError(reference_update=git_ref_error)

        raise NotImplementedError("unexpected variant")  # pragma no cover

    def git_error_normalizer(git_error):
        if not git_error.HasField('reference_update'):
            return

        # `new_oid` in reference update error is about the merge commit,
        # already created, before moving the Git branch to it.
        # There is nothing alike on
        # the Mercurial side (unless we implement the same logic for bookmarks
        # but that seems unlikely)
        git_error.reference_update.new_oid = ''

    def assert_compare_errors(same_details=True, **hg_kwargs):
        rpc_helper.apply_request_defaults(hg_kwargs)
        git_kwargs = rpc_helper.request_kwargs_to_git(hg_kwargs)
        git_kwargs.setdefault('repository', comparison.gitaly_repo)

        with pytest.raises(RpcError) as exc_info_hg:
            rpc_helper.rpc('hg', **hg_kwargs)
        with pytest.raises(RpcError) as exc_info_git:
            git_merge(**git_kwargs)
        rpc_helper.assert_compare_grpc_exceptions(
            exc_info_hg.value,
            exc_info_git.value,
            structured_errors_handler=dict(hg_cls=MergeBranchError,
                                           git_cls=UserMergeBranchError,
                                           git_normalizer=git_error_normalizer,
                                           to_git=error_to_git),
            same_details=same_details
        )

    gl_branch = b'branch/default'
    ctx0 = wrapper.write_commit('foo')
    sha0 = ctx0.hex()
    wrapper.write_commit('foo').hex()
    ctx2 = wrapper.write_commit('zoo', topic='sampletop')
    sha2 = ctx2.hex()
    ctx_conflict = wrapper.write_commit('foo', parent=ctx0, topic='conflict')

    # Conflict
    assert_compare_errors(branch=gl_branch,
                          commit_id=ctx_conflict.hex().decode(),
                          message=b"Will not merge")

    # revision resolution errors
    unknown_sha = '2134cafe' * 5
    assert_compare_errors(commit_id=unknown_sha,
                          expected_old_oid=sha0.decode(),
                          branch=gl_branch)
    assert_compare_errors(commit_id='not-an-id',
                          same_details=False,
                          expected_old_oid=sha0.decode(),
                          branch=gl_branch)
    assert_compare_errors(commit_id=sha2,
                          same_details=False,
                          expected_old_oid='not-an-id',
                          branch=gl_branch)
    assert_compare_errors(commit_id=sha2,
                          expected_old_oid=unknown_sha,
                          branch=gl_branch)

    # expected_old_oid mismatch
    assert_compare_errors(commit_id=sha2,
                          expected_old_oid=sha0,
                          message=b"msg",
                          same_details=True,
                          branch=gl_branch)

    # errors on missing arguments
    assert_compare_errors(commit_id=sha2)
    assert_compare_errors(user=None, branch=gl_branch, commit_id=sha2)

    # Conflict
    assert_compare_errors(commit_id=ctx_conflict.hex().decode(),
                          branch=gl_branch)



# Copyright 2020 Georges Racinet <georges.racinet@octobus.net>
#
# This software may be used and distributed according to the terms of the
# GNU General Public License version 2 or any later version.
#
# SPDX-License-Identifier: GPL-2.0-or-later
import os
import pytest

from mercurial import (
    error,
)
from heptapod.testhelpers import (
    make_ui,
    LocalRepoWrapper,
)

import hgext3rd.hgitaly as hgitaly_ext
HGEXT_HGITALY_SOURCE = os.path.dirname(hgitaly_ext.__file__)

DEFAULT_LISTEN_URL_STR = hgitaly_ext.DEFAULT_LISTEN_URL.decode()


def activating_config():
    return dict(extensions=dict(hgitaly=HGEXT_HGITALY_SOURCE))


def recorder(recorders):
    def inner(*args, **kwargs):
        recorders.append((args, kwargs))

    return inner


def mock_run_forever(tmpdir, monkeypatch, config=None):
    """Uses the monkeypatch fixture to mock `hgitaly.server.run_forever`

    :return: ``ui, records`` where ``records`` is a list that will receive
       a pair `(args, kwargs)` for each call of the  mocked `run_forever`
    """
    # using RepoWrapper is pure lazyness on our part: they  give us the easiest
    # access to fully set up `ui` objects, with activated extensions
    if config is not None:
        # simple merge logic
        for section, items in config.items():
            config.setdefault(section, {}).update(items)
    ui = LocalRepoWrapper.init(tmpdir, config=config).repo.ui
    records = []
    monkeypatch.setattr(hgitaly_ext, 'run_forever', recorder(records))
    return ui, records


def test_serve_default_url(tmpdir, monkeypatch):
    ui, records = mock_run_forever(tmpdir, monkeypatch)
    hgitaly_ext.serve(ui, listen=[], repositories_root=b'/some/path')
    (listen_urls, storages), kwargs = records[0]

    # we have no bytes in the inner call
    assert listen_urls == [DEFAULT_LISTEN_URL_STR]
    assert storages == dict(default=b'/some/path')

    # default for workers is also expected to be ``None``
    # (be it explicitely passed or not)
    assert kwargs.get('nb_workers') is None


def test_serve_workers_config(tmpdir, monkeypatch):
    ui, records = mock_run_forever(tmpdir, monkeypatch,
                                   config=dict(hgitaly=dict(workers='47')))
    hgitaly_ext.serve(ui, listen=[], repositories_root=b'/some/path')
    kwargs = records[0][1]
    assert kwargs.get('nb_workers') == 47


def test_serve_max_rss_mb(tmpdir, monkeypatch):
    ui, records = mock_run_forever(
        tmpdir, monkeypatch,
        config=dict(hgitaly={'worker.max-rss-mib': '128'}))
    hgitaly_ext.serve(ui, listen=[], repositories_root=b'/some/path')
    kwargs = records[0][1]
    assert kwargs.get('max_rss_mib') == 128


def test_serve_monitoring_interval(tmpdir, monkeypatch):
    ui, records = mock_run_forever(
        tmpdir, monkeypatch,
        config=dict(hgitaly={'worker.monitoring-interval-seconds': '13'}))
    hgitaly_ext.serve(ui, listen=[], repositories_root=b'/some/path')
    kwargs = records[0][1]
    assert kwargs.get('monitoring_interval') == 13


def test_serve_graceful_shutdown(tmpdir, monkeypatch):
    ui, records = mock_run_forever(
        tmpdir, monkeypatch,
        config=dict(hgitaly={'worker.graceful-shutdown-timeout-seconds': '18'
                             }))
    hgitaly_ext.serve(ui, listen=[], repositories_root=b'/some/path')
    kwargs = records[0][1]
    assert kwargs.get('graceful_shutdown_timeout_seconds') == 18


def test_serve_config_repositories_root(tmpdir, monkeypatch):
    ui, records = mock_run_forever(tmpdir, monkeypatch)
    ui.setconfig(b'heptapod', b'repositories-root', b'/explicit/path')
    hgitaly_ext.serve(ui, listen=[])
    listen_urls, storages = records[0][0]

    # we have no bytes in the inner call
    assert listen_urls == [DEFAULT_LISTEN_URL_STR]
    assert storages == dict(default=b'/explicit/path')


def test_missing_repos_root():
    ui = make_ui(None, config=activating_config())
    with pytest.raises(error.Abort) as exc_info:
        hgitaly_ext.serve(ui, listen=[])
    assert b'repositories-root' in exc_info.value.args[0]


def test_reraising():
    # the point here is that we don't mock run_forever() , instead
    # we test the whole loop from bad arguments, leading hgitaly.server.init()
    # to raise exceptions and we test the final conversions of the latter.
    ui = make_ui(None, config=activating_config())
    repos_root = b'/some/path'

    invalid_ipv6_url = b'tcp://[::'
    with pytest.raises(error.Abort) as exc_info:
        hgitaly_ext.serve(ui, listen=[invalid_ipv6_url],
                          repositories_root=repos_root)
    message = exc_info.value.args[0]
    assert invalid_ipv6_url in message
    assert b'Invalid IPv6 URL' in message  # explanation from urlparse()

    unsupported_scheme_url = b'exotic://localhost:1234'
    with pytest.raises(error.Abort) as exc_info:
        hgitaly_ext.serve(ui, listen=[unsupported_scheme_url],
                          repositories_root=repos_root)
    message = exc_info.value.args[0]
    assert b"scheme: 'exotic'" in message

    cant_bind_url = b'tcp://unresolvable-or-youre-kidding-me:1234'
    with pytest.raises(error.Abort) as exc_info:
        hgitaly_ext.serve(ui, listen=[cant_bind_url],
                          repositories_root=repos_root)
    message = exc_info.value.args[0]
    assert cant_bind_url in message
    assert b'could not listen' in message



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import ref_pb2 as ref__pb2


class RefServiceStub(object):
    """RefService is a service that provides RPCs to list and modify Git references.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.FindDefaultBranchName = channel.unary_unary(
                '/gitaly.RefService/FindDefaultBranchName',
                request_serializer=ref__pb2.FindDefaultBranchNameRequest.SerializeToString,
                response_deserializer=ref__pb2.FindDefaultBranchNameResponse.FromString,
                )
        self.FindLocalBranches = channel.unary_stream(
                '/gitaly.RefService/FindLocalBranches',
                request_serializer=ref__pb2.FindLocalBranchesRequest.SerializeToString,
                response_deserializer=ref__pb2.FindLocalBranchesResponse.FromString,
                )
        self.FindAllBranches = channel.unary_stream(
                '/gitaly.RefService/FindAllBranches',
                request_serializer=ref__pb2.FindAllBranchesRequest.SerializeToString,
                response_deserializer=ref__pb2.FindAllBranchesResponse.FromString,
                )
        self.FindAllTags = channel.unary_stream(
                '/gitaly.RefService/FindAllTags',
                request_serializer=ref__pb2.FindAllTagsRequest.SerializeToString,
                response_deserializer=ref__pb2.FindAllTagsResponse.FromString,
                )
        self.FindTag = channel.unary_unary(
                '/gitaly.RefService/FindTag',
                request_serializer=ref__pb2.FindTagRequest.SerializeToString,
                response_deserializer=ref__pb2.FindTagResponse.FromString,
                )
        self.FindAllRemoteBranches = channel.unary_stream(
                '/gitaly.RefService/FindAllRemoteBranches',
                request_serializer=ref__pb2.FindAllRemoteBranchesRequest.SerializeToString,
                response_deserializer=ref__pb2.FindAllRemoteBranchesResponse.FromString,
                )
        self.RefExists = channel.unary_unary(
                '/gitaly.RefService/RefExists',
                request_serializer=ref__pb2.RefExistsRequest.SerializeToString,
                response_deserializer=ref__pb2.RefExistsResponse.FromString,
                )
        self.FindBranch = channel.unary_unary(
                '/gitaly.RefService/FindBranch',
                request_serializer=ref__pb2.FindBranchRequest.SerializeToString,
                response_deserializer=ref__pb2.FindBranchResponse.FromString,
                )
        self.UpdateReferences = channel.stream_unary(
                '/gitaly.RefService/UpdateReferences',
                request_serializer=ref__pb2.UpdateReferencesRequest.SerializeToString,
                response_deserializer=ref__pb2.UpdateReferencesResponse.FromString,
                )
        self.DeleteRefs = channel.unary_unary(
                '/gitaly.RefService/DeleteRefs',
                request_serializer=ref__pb2.DeleteRefsRequest.SerializeToString,
                response_deserializer=ref__pb2.DeleteRefsResponse.FromString,
                )
        self.ListBranchNamesContainingCommit = channel.unary_stream(
                '/gitaly.RefService/ListBranchNamesContainingCommit',
                request_serializer=ref__pb2.ListBranchNamesContainingCommitRequest.SerializeToString,
                response_deserializer=ref__pb2.ListBranchNamesContainingCommitResponse.FromString,
                )
        self.ListTagNamesContainingCommit = channel.unary_stream(
                '/gitaly.RefService/ListTagNamesContainingCommit',
                request_serializer=ref__pb2.ListTagNamesContainingCommitRequest.SerializeToString,
                response_deserializer=ref__pb2.ListTagNamesContainingCommitResponse.FromString,
                )
        self.GetTagSignatures = channel.unary_stream(
                '/gitaly.RefService/GetTagSignatures',
                request_serializer=ref__pb2.GetTagSignaturesRequest.SerializeToString,
                response_deserializer=ref__pb2.GetTagSignaturesResponse.FromString,
                )
        self.GetTagMessages = channel.unary_stream(
                '/gitaly.RefService/GetTagMessages',
                request_serializer=ref__pb2.GetTagMessagesRequest.SerializeToString,
                response_deserializer=ref__pb2.GetTagMessagesResponse.FromString,
                )
        self.ListRefs = channel.unary_stream(
                '/gitaly.RefService/ListRefs',
                request_serializer=ref__pb2.ListRefsRequest.SerializeToString,
                response_deserializer=ref__pb2.ListRefsResponse.FromString,
                )
        self.FindRefsByOID = channel.unary_unary(
                '/gitaly.RefService/FindRefsByOID',
                request_serializer=ref__pb2.FindRefsByOIDRequest.SerializeToString,
                response_deserializer=ref__pb2.FindRefsByOIDResponse.FromString,
                )


class RefServiceServicer(object):
    """RefService is a service that provides RPCs to list and modify Git references.
    """

    def FindDefaultBranchName(self, request, context):
        """FindDefaultBranchName looks up the default branch reference name. Unless
        otherwise specified the following heuristic is used:

        1. If there are no branches, return an empty string.
        2. If there is only one branch, return the only branch.
        3. If a branch exists that matches HEAD, return the HEAD reference name.
        4. If a branch exists named refs/heads/main, return refs/heads/main.
        5. If a branch exists named refs/heads/master, return refs/heads/master.
        6. Return the first branch (as per default ordering by git).
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def FindLocalBranches(self, request, context):
        """FindLocalBranches finds all the local branches under `refs/heads/` for the specified repository.
        The response is streamed back to the client to divide the list of branches into chunks.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def FindAllBranches(self, request, context):
        """FindAllBranches finds all branches under `refs/heads/` and `refs/remotes/` for the specified repository.
        The response is streamed back to the client to divide the list of branches into chunks.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def FindAllTags(self, request, context):
        """FindAllTags finds all tags under `refs/tags/` for the specified repository.
        The response is streamed back to the client to divide the list of tags into chunks.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def FindTag(self, request, context):
        """FindTag looks up a tag by its name and returns it to the caller if it exists. This RPC supports
        both lightweight and annotated tags. Note: this RPC returns an `Internal` error if the tag was
        not found.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def FindAllRemoteBranches(self, request, context):
        """FindAllRemoteBranches finds all the remote branches under `refs/remotes/` for the specified repository.
        The response is streamed back to the client to divide the list of branches into chunks.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RefExists(self, request, context):
        """RefExists checks if the specified reference exists. The reference must be fully qualified.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def FindBranch(self, request, context):
        """FindBranch finds a branch by its unqualified name (like "master") and
        returns the commit it currently points to.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdateReferences(self, request_iterator, context):
        """UpdateReferences atomically updates a set of references to a new state. This RPC allows creating
        new references, deleting old references and updating existing references in a raceless way.

        Updating symbolic references with this RPC is not allowed.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteRefs(self, request, context):
        """DeleteRefs deletes the specified references from its repository. Attempting to delete an
        non-existent reference does not result in an error. It is recommended to instead use the
        UpdateReferences RPC because it can delete references in a raceless manner via the expected old
        object ID.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListBranchNamesContainingCommit(self, request, context):
        """ListBranchNamesContainingCommit finds all branches under `refs/heads/` that contain the specified commit.
        The response is streamed back to the client to divide the list of branches into chunks.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListTagNamesContainingCommit(self, request, context):
        """ListTagNamesContainingCommit finds all tags under `refs/tags/` that contain the specified commit.
        The response is streamed back to the client to divide the list of tags into chunks.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTagSignatures(self, request, context):
        """GetTagSignatures returns signatures for annotated tags resolved from a set of revisions. Revisions
        which don't resolve to an annotated tag are silently discarded. Revisions which cannot be resolved
        result in an error. Tags which are annotated but not signed will return a TagSignature response
        which has no signature, but its unsigned contents will still be returned.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTagMessages(self, request, context):
        """GetTagMessages returns tag messages for the annotated tags identified via the given revisions.
        The response is streamed back to the client with a response message containing the tag ID
        always preceding one or more messages containing the tag message contents. This is repeated for
        all tags in the response.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListRefs(self, request, context):
        """ListRefs returns a stream of all references in the repository. By default, pseudo-revisions like HEAD
        will not be returned by this RPC. Any symbolic references will be resolved to the object ID it is
        pointing at.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def FindRefsByOID(self, request, context):
        """FindRefsByOID returns an array of fully qualified reference names that point to an object ID.
        It returns nothing if the object ID doesn't exist, or doesn't point to
        any branches or tags. Prefixes can be also be used as the object ID.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_RefServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'FindDefaultBranchName': grpc.unary_unary_rpc_method_handler(
                    servicer.FindDefaultBranchName,
                    request_deserializer=ref__pb2.FindDefaultBranchNameRequest.FromString,
                    response_serializer=ref__pb2.FindDefaultBranchNameResponse.SerializeToString,
            ),
            'FindLocalBranches': grpc.unary_stream_rpc_method_handler(
                    servicer.FindLocalBranches,
                    request_deserializer=ref__pb2.FindLocalBranchesRequest.FromString,
                    response_serializer=ref__pb2.FindLocalBranchesResponse.SerializeToString,
            ),
            'FindAllBranches': grpc.unary_stream_rpc_method_handler(
                    servicer.FindAllBranches,
                    request_deserializer=ref__pb2.FindAllBranchesRequest.FromString,
                    response_serializer=ref__pb2.FindAllBranchesResponse.SerializeToString,
            ),
            'FindAllTags': grpc.unary_stream_rpc_method_handler(
                    servicer.FindAllTags,
                    request_deserializer=ref__pb2.FindAllTagsRequest.FromString,
                    response_serializer=ref__pb2.FindAllTagsResponse.SerializeToString,
            ),
            'FindTag': grpc.unary_unary_rpc_method_handler(
                    servicer.FindTag,
                    request_deserializer=ref__pb2.FindTagRequest.FromString,
                    response_serializer=ref__pb2.FindTagResponse.SerializeToString,
            ),
            'FindAllRemoteBranches': grpc.unary_stream_rpc_method_handler(
                    servicer.FindAllRemoteBranches,
                    request_deserializer=ref__pb2.FindAllRemoteBranchesRequest.FromString,
                    response_serializer=ref__pb2.FindAllRemoteBranchesResponse.SerializeToString,
            ),
            'RefExists': grpc.unary_unary_rpc_method_handler(
                    servicer.RefExists,
                    request_deserializer=ref__pb2.RefExistsRequest.FromString,
                    response_serializer=ref__pb2.RefExistsResponse.SerializeToString,
            ),
            'FindBranch': grpc.unary_unary_rpc_method_handler(
                    servicer.FindBranch,
                    request_deserializer=ref__pb2.FindBranchRequest.FromString,
                    response_serializer=ref__pb2.FindBranchResponse.SerializeToString,
            ),
            'UpdateReferences': grpc.stream_unary_rpc_method_handler(
                    servicer.UpdateReferences,
                    request_deserializer=ref__pb2.UpdateReferencesRequest.FromString,
                    response_serializer=ref__pb2.UpdateReferencesResponse.SerializeToString,
            ),
            'DeleteRefs': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteRefs,
                    request_deserializer=ref__pb2.DeleteRefsRequest.FromString,
                    response_serializer=ref__pb2.DeleteRefsResponse.SerializeToString,
            ),
            'ListBranchNamesContainingCommit': grpc.unary_stream_rpc_method_handler(
                    servicer.ListBranchNamesContainingCommit,
                    request_deserializer=ref__pb2.ListBranchNamesContainingCommitRequest.FromString,
                    response_serializer=ref__pb2.ListBranchNamesContainingCommitResponse.SerializeToString,
            ),
            'ListTagNamesContainingCommit': grpc.unary_stream_rpc_method_handler(
                    servicer.ListTagNamesContainingCommit,
                    request_deserializer=ref__pb2.ListTagNamesContainingCommitRequest.FromString,
                    response_serializer=ref__pb2.ListTagNamesContainingCommitResponse.SerializeToString,
            ),
            'GetTagSignatures': grpc.unary_stream_rpc_method_handler(
                    servicer.GetTagSignatures,
                    request_deserializer=ref__pb2.GetTagSignaturesRequest.FromString,
                    response_serializer=ref__pb2.GetTagSignaturesResponse.SerializeToString,
            ),
            'GetTagMessages': grpc.unary_stream_rpc_method_handler(
                    servicer.GetTagMessages,
                    request_deserializer=ref__pb2.GetTagMessagesRequest.FromString,
                    response_serializer=ref__pb2.GetTagMessagesResponse.SerializeToString,
            ),
            'ListRefs': grpc.unary_stream_rpc_method_handler(
                    servicer.ListRefs,
                    request_deserializer=ref__pb2.ListRefsRequest.FromString,
                    response_serializer=ref__pb2.ListRefsResponse.SerializeToString,
            ),
            'FindRefsByOID': grpc.unary_unary_rpc_method_handler(
                    servicer.FindRefsByOID,
                    request_deserializer=ref__pb2.FindRefsByOIDRequest.FromString,
                    response_serializer=ref__pb2.FindRefsByOIDResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'gitaly.RefService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class RefService(object):
    """RefService is a service that provides RPCs to list and modify Git references.
    """

    @staticmethod
    def FindDefaultBranchName(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/gitaly.RefService/FindDefaultBranchName',
            ref__pb2.FindDefaultBranchNameRequest.SerializeToString,
            ref__pb2.FindDefaultBranchNameResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def FindLocalBranches(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/gitaly.RefService/FindLocalBranches',
            ref__pb2.FindLocalBranchesRequest.SerializeToString,
            ref__pb2.FindLocalBranchesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def FindAllBranches(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/gitaly.RefService/FindAllBranches',
            ref__pb2.FindAllBranchesRequest.SerializeToString,
            ref__pb2.FindAllBranchesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def FindAllTags(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/gitaly.RefService/FindAllTags',
            ref__pb2.FindAllTagsRequest.SerializeToString,
            ref__pb2.FindAllTagsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def FindTag(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/gitaly.RefService/FindTag',
            ref__pb2.FindTagRequest.SerializeToString,
            ref__pb2.FindTagResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def FindAllRemoteBranches(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/gitaly.RefService/FindAllRemoteBranches',
            ref__pb2.FindAllRemoteBranchesRequest.SerializeToString,
            ref__pb2.FindAllRemoteBranchesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RefExists(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/gitaly.RefService/RefExists',
            ref__pb2.RefExistsRequest.SerializeToString,
            ref__pb2.RefExistsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def FindBranch(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/gitaly.RefService/FindBranch',
            ref__pb2.FindBranchRequest.SerializeToString,
            ref__pb2.FindBranchResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UpdateReferences(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_unary(request_iterator, target, '/gitaly.RefService/UpdateReferences',
            ref__pb2.UpdateReferencesRequest.SerializeToString,
            ref__pb2.UpdateReferencesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteRefs(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/gitaly.RefService/DeleteRefs',
            ref__pb2.DeleteRefsRequest.SerializeToString,
            ref__pb2.DeleteRefsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListBranchNamesContainingCommit(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/gitaly.RefService/ListBranchNamesContainingCommit',
            ref__pb2.ListBranchNamesContainingCommitRequest.SerializeToString,
            ref__pb2.ListBranchNamesContainingCommitResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListTagNamesContainingCommit(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/gitaly.RefService/ListTagNamesContainingCommit',
            ref__pb2.ListTagNamesContainingCommitRequest.SerializeToString,
            ref__pb2.ListTagNamesContainingCommitResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTagSignatures(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/gitaly.RefService/GetTagSignatures',
            ref__pb2.GetTagSignaturesRequest.SerializeToString,
            ref__pb2.GetTagSignaturesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTagMessages(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/gitaly.RefService/GetTagMessages',
            ref__pb2.GetTagMessagesRequest.SerializeToString,
            ref__pb2.GetTagMessagesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListRefs(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/gitaly.RefService/ListRefs',
            ref__pb2.ListRefsRequest.SerializeToString,
            ref__pb2.ListRefsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def FindRefsByOID(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/gitaly.RefService/FindRefsByOID',
            ref__pb2.FindRefsByOIDRequest.SerializeToString,
            ref__pb2.FindRefsByOIDResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# Copyright 2021 Georges Racinet <georges.racinet@octobus.net>
#
# This software may be used and distributed according to the terms of the
# GNU General Public License version 2 or any later version.
#
# SPDX-License-Identifier: GPL-2.0-or-later
import grpc
from importlib import reload
import os
import pytest

from mercurial import (
    phases,
    sshpeer,
)
from hgext3rd.heptapod.branch import (
    write_gitlab_branches,
)


from hgitaly.testing.ssh import hg_exe_path
from hgitaly.testing.sshd import hg_ssh_setup
from hgitaly.tests.common import make_empty_repo

from hgitaly.stub.mercurial_repository_pb2 import (
    ConfigItemType,
    GetConfigItemRequest,
    GetManagedConfigRequest,
    SetManagedConfigRequest,
    HeptapodConfigSection,
    MercurialPeer,
    PushRequest,
)
from hgitaly.stub.mercurial_repository_pb2_grpc import (
    MercurialRepositoryServiceStub,
)

from .fixture import ServiceFixture

AutoPublish = HeptapodConfigSection.AutoPublish

parametrize = pytest.mark.parametrize


class ConfigFixture(ServiceFixture):
    stub_cls = MercurialRepositoryServiceStub

    def config_item(self, **kw):
        return self.stub.GetConfigItem(
            GetConfigItemRequest(repository=self.grpc_repo, **kw))

    def config_bool(self, section, name):
        return self.config_item(as_type=ConfigItemType.BOOL,
                                section=section,
                                name=name,
                                ).as_bool

    def get_managed_config(self, **kw):
        return self.stub.GetManagedConfig(
            GetManagedConfigRequest(repository=self.grpc_repo, **kw))

    def set_managed_config(self, **kw):
        return self.stub.SetManagedConfig(
            SetManagedConfigRequest(repository=self.grpc_repo, **kw))

    def hgrc_path(self, main=False):
        hgrc_name = 'hgrc' if main else 'hgrc.managed'
        return self.repo_wrapper.path / '.hg' / hgrc_name

    def write_hgrc(self, *lines, main=False):
        self.hgrc_path().write_text('\n'.join((lines)))

    def append_main_hgrc(self, *lines):
        with open(self.hgrc_path(main=True), 'a') as hgrcf:
            hgrcf.write('\n'.join(lines))

    def write_main_hgrc(self, *lines, include_managed=True):
        with open(self.hgrc_path(main=True), 'w') as hgrcf:
            hgrcf.write('\n'.join(lines))
        if include_managed:
            self.include_managed_hgrc()

    def read_main_hgrc_lines(self):
        return self.hgrc_path(main=True).read_text().splitlines()

    def include_managed_hgrc(self):
        self.append_main_hgrc('', '%include hgrc.managed', '')


@pytest.fixture
def config_fixture(grpc_channel, server_repos_root):
    with ConfigFixture(grpc_channel, server_repos_root) as fixture:
        # this is normally done upon repository creation:
        fixture.include_managed_hgrc()
        setattr(fixture.grpc_repo, 'gl_project_path', 'mygroup/myproject')

        yield fixture


def test_config_item(config_fixture):
    rpc_config_bool = config_fixture.config_bool

    # Relying on current defaults in some random core settings, just pick
    # some other ones if they change.
    assert rpc_config_bool('format', 'usestore') is True
    assert rpc_config_bool('commands', 'status.verbose') is False


def test_get_managed_config(config_fixture):
    get_config = config_fixture.get_managed_config

    # first, default values
    config = get_config()
    # in these tests, the main hgrc is not initialized with the default
    # group inheritance.
    assert config.inherit is False

    section = config.heptapod
    assert section.auto_publish == AutoPublish.WITHOUT_TOPIC

    config_fixture.write_hgrc('[heptapod]',
                              'auto-publish = nothing',
                              'allow-bookmarks = yes',
                              '')
    section = get_config().heptapod
    assert section.auto_publish == AutoPublish.NOTHING
    assert section.allow_bookmarks is True

    config_fixture.append_main_hgrc('\n%include ../../../../../mygroup/hgrc\n')
    assert get_config().inherit is True


def test_get_managed_config_local(config_fixture):
    get_config = config_fixture.get_managed_config
    # Instead of inheriting from group (which Heptapod generally would do),
    # in this test, we set a direct value in the main HGRC file.
    # With 'local=True`, it should not be returned by the RPC either.

    config_fixture.append_main_hgrc('', '[heptapod]', 'auto-publish=all', '')

    config_fixture.write_hgrc('[heptapod]',
                              'allow-multiple-heads = yes',
                              '')
    section = get_config(local=True).heptapod
    # local call does not see the value from main HGRC, hence the default
    # value of the protocol is to be seen
    assert section.auto_publish == AutoPublish.WITHOUT_TOPIC
    # but parameters not present in the managed file are simply not set
    # (field is optional in protocol)
    assert not section.HasField('auto_publish')
    assert not section.HasField('allow_bookmarks')

    # and that is not because it is totally blind:
    assert section.allow_multiple_heads is True

    # double check
    assert get_config(local=False).heptapod.auto_publish == AutoPublish.ALL


def test_set_managed_config(config_fixture):
    get_config = config_fixture.get_managed_config
    set_config = config_fixture.set_managed_config

    config_fixture.write_main_hgrc('[heptapod]',
                                   'allow-multiple-heads = yes',
                                   '')
    set_config(
        heptapod=HeptapodConfigSection(allow_bookmarks=True,
                                       auto_publish=AutoPublish.NOTHING,
                                       ),
        by_line="by user foo"
    )
    section = get_config(local=True).heptapod
    assert not section.HasField('allow_multiple_heads')
    assert section.HasField('allow_bookmarks')
    assert section.allow_bookmarks
    assert section.HasField('auto_publish')
    assert section.auto_publish == AutoPublish.NOTHING
    assert get_config().heptapod.allow_multiple_heads is True  # control

    # to check whether the managed file actually overrides a setting
    # and even if inclusion is missing
    config_fixture.write_main_hgrc('[heptapod]', 'allow-bookmarks = yes', '',
                                   include_managed=False)

    set_config(heptapod=HeptapodConfigSection(allow_bookmarks=False))
    section = get_config(local=True).heptapod
    assert not section.HasField('allow_multiple_heads')
    assert section.HasField('allow_bookmarks')
    assert not section.allow_bookmarks
    # field not mentioned in call is not affected
    assert section.HasField('auto_publish')
    assert section.auto_publish == AutoPublish.NOTHING
    # proof of override
    assert not get_config().heptapod.allow_bookmarks

    set_config(heptapod=HeptapodConfigSection(),
               remove_items=['allow_bookmarks'])
    section = get_config(local=True).heptapod
    assert not section.HasField('allow_multiple_heads')
    assert not section.HasField('allow_bookmarks')
    assert not section.allow_multiple_heads
    assert not section.allow_bookmarks
    # proof that override has been removed
    assert get_config().heptapod.allow_bookmarks

    # removing everything and testing by_line
    set_config(remove_items=('allow_bookmarks', 'auto_publish'),
               by_line='by erasor')
    managed_lines = config_fixture.hgrc_path().read_text().splitlines()
    assert not managed_lines[0].startswith('[')
    assert managed_lines[1:] == ['# latest update by erasor', '']


def test_set_managed_config_inherit(config_fixture):
    get_config = config_fixture.get_managed_config
    set_config = config_fixture.set_managed_config
    config_fixture.write_main_hgrc("# An unrelated line",
                                   "%include some/path/hgrc",
                                   "# Another unrelated line",
                                   '')
    assert get_config().inherit is True

    set_config(inherit=False, by_line="by user foo1")
    assert get_config().inherit is False
    hgrc_lines = config_fixture.read_main_hgrc_lines()
    assert hgrc_lines[:3] == ["# An unrelated line",
                              "# inheritance removed by user foo1",
                              "# Another unrelated line",
                              ]
    set_config(inherit=True, by_line="by user foo2")
    assert get_config().inherit is True
    hgrc_lines = config_fixture.read_main_hgrc_lines()
    assert hgrc_lines[:5] == ["# inheritance restored by user foo2",
                              "%include ../../mygroup/hgrc",
                              "# An unrelated line",
                              "# inheritance removed by user foo1",
                              "# Another unrelated line",
                              ]

    # in case of no-op, nothing is changed (even by-line)
    set_config(inherit=True, by_line="by user foo3")
    assert get_config().inherit is True
    hgrc_lines = config_fixture.read_main_hgrc_lines()
    assert hgrc_lines[:2] == ["# inheritance restored by user foo2",
                              "%include ../../mygroup/hgrc",
                              ]


@pytest.fixture
def push_fixture(grpc_channel, server_repos_root):
    hg_repo_stub = MercurialRepositoryServiceStub(grpc_channel)
    wrapper, grpc_repo = make_empty_repo(server_repos_root)
    target_wrapper, _target_grpc_repo = make_empty_repo(server_repos_root)

    # make sure that the sshpeer.sshv1peer class inherits from the
    # `wirepeer` patched as `tnspeer` by `hgext3rd.topic.server`
    reload(sshpeer)

    def push(remote_url=target_wrapper.path, peer_opts=None, **kw):
        if peer_opts is None:
            peer_opts = {}
        return hg_repo_stub.Push(PushRequest(
            repository=grpc_repo,
            remote_peer=MercurialPeer(url=str(remote_url), **peer_opts),
            **kw))
    yield wrapper, target_wrapper, push


def auto_publish_config(publish):
    """Return a config dict with phases.publish set."""
    return dict(phases=dict(publish='yes' if publish else 'no'))


def set_auto_publishing(repo_wrapper, publish):
    """Tweak persistently the auto-publishing behaviour of the repository

    In a typical push, the repository configuration will be initialized
    independently from the objects of these tests, so this has to be done
    in the `hgrc` file.

    TODO consider for inclusion in mercurial-testhelpers.
    """
    repo_wrapper.write_hgrc(auto_publish_config(publish))


def test_bare_push(push_fixture):
    wrapper, target_wrapper, push = push_fixture

    def get_nodes_ids(repo):
        return {repo[r].hex() for r in repo.revs("all()")}

    ctx_pub = wrapper.commit_file('foo')
    wrapper.set_phase("public", [ctx_pub.hex()])
    ctx_draft = wrapper.commit_file('foo')

    # pushing public changesets only
    res = push()
    assert res.new_changesets
    target_wrapper.reload()
    target_node_ids = get_nodes_ids(target_wrapper.repo)
    assert target_node_ids == {ctx_pub.hex()}

    # idempotency
    res = push()
    assert not res.new_changesets

    # pushing drafts explicitely
    # this needs us to make the target repo non-publishing
    set_auto_publishing(target_wrapper, False)
    res = push(include_drafts=True)
    assert res.new_changesets
    target_wrapper.reload()
    target_node_ids = get_nodes_ids(target_wrapper.repo)
    assert target_node_ids == {ctx_pub.hex(), ctx_draft.hex()}

    # idempotency for drafts
    res = push(include_drafts=True)
    assert not res.new_changesets

    # case of target not existing
    with pytest.raises(grpc.RpcError) as exc_info:
        push(remote_url=target_wrapper.path / 'does/not/exist')
    assert exc_info.value.code() == grpc.StatusCode.INVALID_ARGUMENT
    assert "could not be used as a peer" in exc_info.value.details()

    # case of push failing on target (other ways to make it fail would
    # be of course admissible)
    (target_wrapper.path / '.hg' / 'hgrc').write_text(
        "[experimental]\n"
        "single-head-per-branch=yes\n")
    wrapper.write_commit('bar', parent=ctx_pub)
    with pytest.raises(grpc.RpcError) as exc_info:
        push(remote_url=target_wrapper.path, include_drafts=True)
    assert exc_info.value.code() == grpc.StatusCode.INTERNAL


def test_push_branch(push_fixture):
    wrapper, target_wrapper, push = push_fixture

    def get_node_ids(wrapper, revset="all()"):
        repo = wrapper.repo
        return {repo[r].hex() for r in repo.revs(revset)}

    ctx_foo = wrapper.commit_file('foo')
    wrapper.commit_file('baz')
    wrapper.update(0)
    ctx_bar = wrapper.commit_file('bar', branch="stable")
    wrapper.set_phase("public", [ctx_bar.hex()])
    expected_node_ids = {ctx_foo.hex(), ctx_bar.hex()}
    res = push(only_gitlab_branches_matching=[b'branch/stable'])

    assert res.new_changesets
    target_wrapper.reload()
    target_node_ids = get_node_ids(target_wrapper)
    assert target_node_ids == expected_node_ids

    # a head draft is not pushed, but its public ancestors are
    # (also using a wildcard pattern)
    ctx_bar2 = wrapper.commit_file('bar', branch='stable')
    ctx_bar3 = wrapper.commit_file('bar', branch='stable')
    wrapper.set_phase('public', [ctx_bar2.hex()])
    res = push(only_gitlab_branches_matching=[b'branch/st*'])
    assert res.new_changesets
    target_wrapper.reload()
    assert get_node_ids(target_wrapper, revset="stable") == {ctx_bar2.hex()}

    # pushing drafts explicitely
    # auto-publication is prohibited for now
    set_auto_publishing(target_wrapper, False)
    res = push(only_gitlab_branches_matching=[b'branch/stable'],
               include_drafts=True)
    assert res.new_changesets
    target_wrapper.reload()
    assert get_node_ids(target_wrapper, revset="stable") == {ctx_bar3.hex()}

    # pushing a branch that doesn't exist currently is not an error
    # (it can happen temporarily and still be the intent in repeated
    # invocations, such as mirrorring, the primary use-case)
    assert not push(only_gitlab_branches_matching=[b'z*']).new_changesets


@pytest.fixture
def ssh_fixture(tmpdir):
    working_dir = tmpdir / 'sshd'
    working_dir.mkdir()
    yield from hg_ssh_setup(working_dir)


def test_push_ssh(push_fixture, ssh_fixture):
    wrapper, target_wrapper, push = push_fixture
    server, client_key, known_hosts = ssh_fixture

    ctx = wrapper.commit_file('foo')

    # auto-publication is prohibited for now
    set_auto_publishing(target_wrapper, False)
    res = push(
        include_drafts=True,
        remote_url=f'ssh://{server.host}:{server.port}/{target_wrapper.path}',
        peer_opts=dict(
            ssh_remote_command=os.fsencode(hg_exe_path()),
            ssh_key=client_key,
            ssh_known_hosts=known_hosts,
        ))

    assert res.new_changesets

    target_wrapper.reload()
    assert ctx in target_wrapper.repo


@parametrize('order', ['hg-evolve-first', 'hg-evolve-second'])
def test_push_ssh_different_capabilities(order,
                                         server_repos_root,
                                         push_fixture,
                                         ssh_fixture):
    wrapper, target_wrapper, push = push_fixture
    server, client_key, known_hosts = ssh_fixture

    target2_wrapper, _ = make_empty_repo(server_repos_root)

    wrapper.commit_file('foo')
    topical = wrapper.commit_file('top', topic='feature')
    peer_opts = dict(
        ssh_remote_command=os.fsencode(hg_exe_path()),
        ssh_key=client_key,
        ssh_known_hosts=known_hosts,
    )

    # configuration for first target, with evolve/topic
    target_conf = auto_publish_config(False)
    target_conf['extensions'] = dict(topic='', evolve='')
    target_wrapper.write_hgrc(target_conf)

    # configuration for second target, without evolve/topic
    target2_conf = auto_publish_config(False)
    target2_conf['extensions'] = dict(topic='!', evolve='!')
    target2_wrapper.write_hgrc(target2_conf)

    ssh_base_url = f'ssh://{server.host}:{server.port}/'
    target_paths = [target_wrapper.path, target2_wrapper.path]
    if order == 'hg-evolve-second':
        target_paths.reverse()

    # if the topic extension calls `tns_heads()` method on a remote that does
    # not have the capability, we get a hard error.
    # As of this writing, the topic extension adds the `tns_heads()`
    # method inconditionally on `mercurial.sshpeer.sshv1peer`
    # but actual call of `tns_heads()` depends on current peer capability, as
    # it should.
    # Sabotaging that latter part (currently `topicpeerexecutor` class, defined
    # in `hgext3rd.topic.server`) breaks this test.
    results = [push(include_drafts=True,
                    remote_url=f'{ssh_base_url}/{tgt_path}',
                    peer_opts=peer_opts)
               for tgt_path in target_paths]

    assert all(res.new_changesets for res in results)

    for tgt_wrapper in (target_wrapper, target2_wrapper):
        tgt_wrapper.reload()
        assert topical in tgt_wrapper.repo


def test_push_auto_publishing_error(push_fixture):
    wrapper, target_wrapper, push = push_fixture

    ctx = wrapper.commit_file('foo')
    assert ctx.phase() == phases.draft  # making sure of main hypothesis

    set_auto_publishing(target_wrapper, True)

    with pytest.raises(grpc.RpcError) as exc_info:
        push(include_drafts=True)
    # debatable, but we currently don't have a clear way to tell the
    # inner exception apart
    assert exc_info.value.code() == grpc.StatusCode.INTERNAL
    assert 'push would publish' in exc_info.value.details()


def test_push_unexpected_error(push_fixture):
    wrapper, target_wrapper, push = push_fixture
    wrapper.commit_file('foo')

    # we don't have many ways to trigger internal errors when
    # monkeypatch can't work (server is not in the same process):
    # let's corrupt the repo.

    write_gitlab_branches(wrapper.repo, {b'wrecked': b'not-a-hash'})

    with pytest.raises(grpc.RpcError) as exc_info:
        push(only_gitlab_branches_matching=[b'wrecked'])

    # debatable, but we currently don't have a clear way to tell the
    # inner exception apart
    assert exc_info.value.code() == grpc.StatusCode.INTERNAL
    details = exc_info.value.details()
    assert "Unexpected" in details
    assert 'not-a-hash' in details


@parametrize('url', (
    'http://.heptapod.test/a/repo',  # hg.peer() fails at IDNA decoding
    'http://\u2100.test',  # refused for NKFC normalisation by urlparse()
    'mailto:pushoversmtpisajoke@heptapod.test',  # wrong scheme
    '/tmp/not_in_storage',
))
def test_push_invalid_url(push_fixture, url):
    _wrapper, _target_wrapper, push = push_fixture
    with pytest.raises(grpc.RpcError) as exc_info:
        push(remote_url=url)
    assert exc_info.value.code() == grpc.StatusCode.INVALID_ARGUMENT



# Copyright 2023 Georges Racinet <georges.racinet@octobus.net>
#
# This software may be used and distributed according to the terms of the
# GNU General Public License version 2 or any later version.
#
# SPDX-License-Identifier: GPL-2.0-or-later

import grpc
import os

# for now, we can simply list all the feature flags that we support
# later on, we'll probably want an automatic extraction: it's not
# hard if we require an either from
# Golang definition or from the Rails app repository. The latter
# is easier, because of YaML format. We could even copy them over.
#
# Note: we see no problem within this Python code to use dash
# as separator. No need to go back and forth between dashes and underscores
# as the Golang implementation does. In Ruby case, the underscores have the
# advantage of being usable as symbols without any quotes.
FEATURE_FLAGS = {  # name -> default value
    'find-tag-structured-error': False,
    'simplify-find-local-branches-response': False,
}
GRPC_PREFIX = 'gitaly-feature-'
ALL_ENABLED = (
    os.environ.get('GITALY_TESTING_ENABLE_ALL_FEATURE_FLAGS') == 'true'
)


class UndefinedError(LookupError):
    """Exception used to qualify query of undefined feature flag.

    This is about HGitaly code querying the value of undefined flag, not
    about incoming requests bearing unknown feature flags.

    In most cases, that means the feature flag has to be defined.
    """


def is_enabled(context: grpc.ServicerContext, name: str) -> bool:
    """Return whether a given feature flag is enabled

    This is meant for HGitaly servicer code.

    :raises UndefinedError: if the feature flag is not defined. This strict
      policy is made possible by the fact that this method is not used to
      validate incoming feature flags (we expect to ignore many of them),
      rather as a HGitaly service method implementation expecting a
      feature flag to exist, given in litteral form. This will break the
      service tests if a feature flag has just been removed and the caller has
      not (yet). With our 100% coverage policy, this is not a hazard for
      production: all calls to `is_enabled` are covered.
    """
    if ALL_ENABLED:
        if name not in FEATURE_FLAGS:
            raise UndefinedError(name)
        return True

    md_key = GRPC_PREFIX + name
    md = dict(context.invocation_metadata())
    val = md.get(md_key)
    if val is None:
        try:
            return FEATURE_FLAGS[name]
        except KeyError:
            raise UndefinedError(name)

    return val == 'true'


def as_grpc_metadata(flag_values):
    if flag_values is None:
        return None

    return [((GRPC_PREFIX + k).encode('ascii'), b'true' if v else b'false')
            for k, v in flag_values]



# Copyright 2020-2022 Georges Racinet <georges.racinet@octobus.net>
#
# This software may be used and distributed according to the terms of the
# GNU General Public License version 2 or any later version.
#
# SPDX-License-Identifier: GPL-2.0-or-later
"""Server startup logic, mono process and mono thread.

Can be used as a standalone server (e.g, for debugging purposes) or as
a worker in a service with multiple processes.

While the exposition as a Mercurial extension is certainly convenient for
many reasons, it is best to separate what is specific to the extension context
and what is just generic gRPC server code.

Note that gRPC will launch several threads no matter what, but only one will
be active at a given time.
"""
from concurrent import futures
import grpc
import logging
import mercurial
import signal
from urllib.parse import urlparse

from ..service.interceptors import (
    RequestLoggerInterceptor,
)

from ..service.analysis import AnalysisServicer
from ..service.blob import BlobServicer
from ..service.commit import CommitServicer
from ..service.ref import RefServicer
from ..service.diff import DiffServicer
from ..service.mercurial_changeset import MercurialChangesetServicer
from ..service.mercurial_operations import MercurialOperationsServicer
from ..service.mercurial_repository import MercurialRepositoryServicer
from ..service.operations import OperationServicer
from ..service.repository import RepositoryServicer
from ..service.server import ServerServicer

from ..stub.analysis_pb2_grpc import add_AnalysisServiceServicer_to_server
from ..stub.blob_pb2_grpc import add_BlobServiceServicer_to_server
from ..stub.commit_pb2_grpc import add_CommitServiceServicer_to_server
from ..stub.ref_pb2_grpc import add_RefServiceServicer_to_server
from ..stub.diff_pb2_grpc import add_DiffServiceServicer_to_server
from ..stub.repository_pb2_grpc import (
    add_RepositoryServiceServicer_to_server
)
from ..stub.operations_pb2_grpc import (
    add_OperationServiceServicer_to_server
)
from ..stub.mercurial_changeset_pb2_grpc import (
    add_MercurialChangesetServiceServicer_to_server
)
from ..stub.mercurial_operations_pb2_grpc import (
    add_MercurialOperationsServiceServicer_to_server
)
from ..stub.mercurial_repository_pb2_grpc import (
    add_MercurialRepositoryServiceServicer_to_server
)
from ..stub.server_pb2_grpc import add_ServerServiceServicer_to_server

from .address import (
    InvalidUrl,
    UnsupportedUrlScheme,
    apply_default_port,
)

logger = logging.getLogger(__name__)


DEFAULT_TCP_PORT = 9237


class BindError(RuntimeError):
    pass


def init(listen_urls, storages):
    """Return server object for given parameters"""

    server_opts = (
        ('grpc.so_reuseport', 1),
        # As of GitLab 14.8.2, this matches the setting used by Rails'
        # Gitlab::GitalyClient.channel_args, hence avoiding the infamous
        # "Too many pings" error. For explanations, see
        # https://github.com/grpc/grpc/blob/master/doc/keepalive.md
        # in which the PARAMETERS_IN_CAPS are apparently to be replaced by the
        # corresponding *values* from https://github.com...
        # .../grpc/grpc/blob/v1.42.x/include/grpc/impl/codegen/grpc_types.h
        ('grpc.http2.min_ping_interval_without_data_ms', 19000),
    )
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=1),
                         interceptors=[RequestLoggerInterceptor(),
                                       ],
                         options=server_opts)
    add_AnalysisServiceServicer_to_server(AnalysisServicer(storages), server)
    add_BlobServiceServicer_to_server(BlobServicer(storages), server)
    add_CommitServiceServicer_to_server(CommitServicer(storages), server)
    add_RefServiceServicer_to_server(RefServicer(storages), server)
    add_DiffServiceServicer_to_server(DiffServicer(storages), server)
    add_MercurialChangesetServiceServicer_to_server(
        MercurialChangesetServicer(storages), server)
    add_MercurialOperationsServiceServicer_to_server(
        MercurialOperationsServicer(storages), server)
    add_MercurialRepositoryServiceServicer_to_server(
        MercurialRepositoryServicer(storages), server)
    add_OperationServiceServicer_to_server(
        OperationServicer(storages), server)
    add_RepositoryServiceServicer_to_server(
        RepositoryServicer(storages), server)
    add_ServerServiceServicer_to_server(ServerServicer(storages), server)

    for url in listen_urls:
        try:
            parsed_url = urlparse(url)
        except ValueError as exc:
            raise InvalidUrl(url, *exc.args)
        try:
            if parsed_url.scheme == 'tcp':
                server.add_insecure_port(apply_default_port(parsed_url.netloc))
            elif parsed_url.scheme == 'unix':
                server.add_insecure_port(url)
            else:
                raise UnsupportedUrlScheme(parsed_url.scheme)
        except RuntimeError:
            raise BindError(url)

    return server


def server_process(worker_id, listen_urls, storages,
                   graceful_shutdown_timeout_seconds=300,
                   mono_process=False,
                   pytest_cov=False
                   ):
    if pytest_cov:
        from pytest_cov.embed import cleanup_on_signal
        cleanup_on_signal(signal.SIGTERM)
        cleanup_on_signal(signal.SIGINT)

    from hgitaly import procutil
    procutil.IS_CHILD_PROCESS = not mono_process

    server = init(listen_urls, storages)
    server.start()
    signal.signal(signal.SIGTERM,
                  lambda *a: server.stop(graceful_shutdown_timeout_seconds))

    logger.info("Server %d started", worker_id)
    logger.debug("Graceful shutdown timeout is %d seconds",
                 graceful_shutdown_timeout_seconds)
    try:
        server.wait_for_termination()
    except mercurial.error.SignalInterrupt:  # pragma no cover
        # here it would be better to catch the Mercurial signal
        # in our servicer layers and raise the clean thing that
        # grpc probably expects.
        # Another possibility would be to have our own handler in
        # the worker process, but that would perhaps prevent Mercurial
        # from doing its own cleaning.
        logger.info("Terminating on explicit signal")



# Copyright 2020 Georges Racinet <georges.racinet@octobus.net>
#
# This software may be used and distributed according to the terms of the
# GNU General Public License version 2 or any later version.
#
# SPDX-License-Identifier: GPL-2.0-or-later
"""Tests for the RHGitalyServer helper

These don't involve RHGitaly and not even Mercurial. The purpose is to
alert when the foundation of all Gitaly Comparison tests with RHGitaly is
broken, so that developers can focus right away on startup issues and the
like.
"""
import pytest

from hgitaly.stub.server_pb2 import (
    ServerInfoRequest,
)
from hgitaly.stub.server_pb2_grpc import ServerServiceStub

from . import skip_comparison_tests
if skip_comparison_tests():  # pragma no cover
    pytestmark = pytest.mark.skip


def test_rhgitaly_channel(rhgitaly_channel):
    server_stub = ServerServiceStub(rhgitaly_channel)
    assert server_stub.ServerInfo(ServerInfoRequest()).server_version



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: lint.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\nlint.proto\x12\x06gitaly\x1a google/protobuf/descriptor.proto\"\xe5\x01\n\x0cOperationMsg\x12*\n\x02op\x18\x01 \x01(\x0e\x32\x1e.gitaly.OperationMsg.Operation\x12/\n\x0bscope_level\x18\x02 \x01(\x0e\x32\x1a.gitaly.OperationMsg.Scope\"D\n\tOperation\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0b\n\x07MUTATOR\x10\x01\x12\x0c\n\x08\x41\x43\x43\x45SSOR\x10\x02\x12\x0f\n\x0bMAINTENANCE\x10\x03\"2\n\x05Scope\x12\x0e\n\nREPOSITORY\x10\x00\x12\x0b\n\x07STORAGE\x10\x02\"\x04\x08\x01\x10\x01*\x06SERVER:6\n\x0bintercepted\x12\x1f.google.protobuf.ServiceOptions\x18\xfe\x82\x05 \x01(\x08:G\n\x07op_type\x12\x1e.google.protobuf.MethodOptions\x18\xff\x82\x05 \x01(\x0b\x32\x14.gitaly.OperationMsg:0\n\x07storage\x12\x1d.google.protobuf.FieldOptions\x18\xe1\xc8\x05 \x01(\x08:3\n\nrepository\x12\x1d.google.protobuf.FieldOptions\x18\xe2\xc8\x05 \x01(\x08::\n\x11target_repository\x12\x1d.google.protobuf.FieldOptions\x18\xe3\xc8\x05 \x01(\x08:>\n\x15\x61\x64\x64itional_repository\x12\x1d.google.protobuf.FieldOptions\x18\xe4\xc8\x05 \x01(\x08\x42\x34Z2gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'lint_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  google_dot_protobuf_dot_descriptor__pb2.ServiceOptions.RegisterExtension(intercepted)
  google_dot_protobuf_dot_descriptor__pb2.MethodOptions.RegisterExtension(op_type)
  google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(storage)
  google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(repository)
  google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(target_repository)
  google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(additional_repository)

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z2gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypb'
  _globals['_OPERATIONMSG']._serialized_start=57
  _globals['_OPERATIONMSG']._serialized_end=286
  _globals['_OPERATIONMSG_OPERATION']._serialized_start=166
  _globals['_OPERATIONMSG_OPERATION']._serialized_end=234
  _globals['_OPERATIONMSG_SCOPE']._serialized_start=236
  _globals['_OPERATIONMSG_SCOPE']._serialized_end=286
# @@protoc_insertion_point(module_scope)
